<?php

/**
 *  Copyright © 2016 Siberian Federal University
 * 
 *  This file is part of YConfs.
 *  
 *  YConfs is free software: you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License,
 *  for details see http://yconfs.sfu-kras.ru/license. 
 */
class MailTask extends ActiveRecord {

    public $id;
    public $conf_id;
    public $emails;
    public $total_count;
    public $sent_count;
    public $creation_date;
    public $completion_date;

    const STATUS_NEW = 'new';
    const STATUS_STARTED = 'started';
    const STATUS_COMPLETED = 'completed';

    public $status = MailTask::STATUS_NEW;
    public $subject;
    public $body;

    const RECIPIENTS_ALL = 'all';
    const RECIPIENTS_APPROVED = 'approved';

    //  all или accepted + email(галочка me)
    //  all or accepted + email(me checkbox)
    public $recipients; 
    public $email_from;
    public $name_from;

    const MAX_MSGS_COUNT = 50;

    public static function model($className = __CLASS__) {
        return parent::model($className);
    }

    public function tableName() {
        return '{{mail_task}}';
    }

    public function getEmailFrom(){
        return $this->email_from;
    }
    
    public function setEmailFrom($value){
        $this->email_from = $value;
    }
    
    public function getNameFrom(){
        return $this->name_from;
    }
    
    public function setNameFrom($value){
        $this->name_from = $value;
    }
    
    public function behaviors() {
        return array(
            'XssFilterBehavior' => array(
                'class' => 'application.behaviors.XssFilterBehavior',
                'attributes' => array('body'),
                'languages' => Yii::app()->params['languages'],
                'allowed_tags' => XssFilter::$ALLOWED_TAGS
            ),
            'FullXssFilterBehavior' => array(
                'class' => 'application.behaviors.XssFilterBehavior',
                'attributes' => array('subject'),
                'languages' => Yii::app()->params['languages'],
                'allowed_tags' => array()
        ));
    }

    protected $_conf = NULL;

    public function conf() {
        if ($this->_conf == NULL) {
            $this->_conf = Conf::model()->findByPk($this->conf_id);
        }
        return $this->_conf;
    }

    public function perform() {
        $success = true;

        if ($this->status == MailTask::STATUS_NEW) {
            $conf = $this->conf();
            $this->status = MailTask::STATUS_STARTED;
            //  сразу сохраняем статус
            //  status is saved
            $this->save();
            Yii::log(Yii::t('mail', 'Mailing with id = {id} started', array('{id}' => $this->id)), 'info', 'mailtask');
            //  сначала составляем список email-ов авторов и записываем в таблицу
            //  at first compound a list of author emails and write it to the database
            $this->emails = '';
            
            $recipients = explode(",", $this->recipients);
            $email = $recipients[1];
            if (!empty($email)) {
                $this->emails = $email;
            };
            $authors = array();
            if ($recipients[0] == MailTask::RECIPIENTS_ALL) {
                $authors = Author::model()->findAllByConf($conf);
            };
            if ($recipients[0] == MailTask::RECIPIENTS_APPROVED) {
                $authors = Author::model()->findApprovedByConf($conf);
            };
            foreach ($authors as $user) {                      
                $this->emails.=','.$user->email;
            };           
            //  проверяем адреса на правильность и
            //  оставляем только уникальные
            //
            //  validating emails and leave unique emails only
            $emails = explode(",", $this->emails);
            $emails = array_unique($emails); 
            $validator= new CEmailValidator();
            $validator->allowEmpty=false;
            $this->total_count = 0;
            $this->sent_count = 0;
            $this->emails='';
            foreach($emails as $email){
                if(!$validator->validateValue($email)){                  
                    continue;
                }; 
                if(!empty($this->emails)){
                    $this->emails.=',';
                };
                $this->emails.=$email;
                $this->total_count++;
            };      
            $this->save();
            return $success;
        };
        if ($this->status == MailTask::STATUS_STARTED) {           
            $mail = new MailMessage();
            $mail->subject = $this->subject;
            $mail->body = $this->body;
            $mail->emailFrom = $this->emailFrom;
            $mail->nameFrom = $this->nameFrom;
            $emails = explode(",", $this->emails);
            //  удаляем адреса на которые уже разосланы письма
            //  remove emails to which letters are already sent
            for ($i = 1; $i <= $this->sent_count; $i++) {
                array_shift($emails);
            };
            //  высылаем на следующую пачку
            //  sending letters to the next group of emails
            $count = 0;
            foreach ($emails as $email) {
                $mail->emailTo = $email;
                if (!$mail->send(false)) {
                    $success = false;
                    Yii::log(Yii::t('mail', 'Message for mail task with id = {id} was NOT sent to email = {email}', array('{id}' => $this->id, '{email}' => $mail->emailTo)), 'error', 'mailtask');
                } else {
                    Yii::log(Yii::t('mail', 'Message for mail task with id = {id} sent to email = {email}', array('{id}' => $this->id, '{email}' => $mail->emailTo)), 'info', 'mailtask');
                };
                $count++;
                if ($count == MailTask::MAX_MSGS_COUNT) {
                    break;
                };
            };
            $this->sent_count+=$count;
            if ($this->sent_count >= $this->total_count) {
                $this->status = MailTask::STATUS_COMPLETED;
                $this->completion_date = time();
                Yii::log(Yii::t('mail', 'Mailing with id = {id} completed', array('{id}' => $this->id)), 'info', 'mailtask');
            };
            $this->save();
        }
        return $success;
    }

    public function recipientsStr() {
        $str = '';
        $recipients = explode(",", $this->recipients);
        $email = $recipients[1];
        if ($recipients[0] == MailTask::RECIPIENTS_ALL) {
            $str = Yii::t('admin', 'all authors');
        }
        if ($recipients[0] == MailTask::RECIPIENTS_APPROVED) {
            $str = Yii::t('admin', 'authors of the approved applications for participation');
        }
        if (!empty($email)) {
            $str.=' + ' . $email;
        }
        return $str;
    }

    public function scopes() {
        return array(
            'new' => array(
                'condition' => "status='new'"
            ),
            'started' => array(
                'condition' => "status='started'"
            )
        );
    }

    public function findAllNew() {
        return $this->new()->findAll();
    }

    public function findAllStarted() {
        return $this->started()->findAll();
    }

    public function findAllByConf($conf) {
        $criteria = new CDbCriteria;
        $criteria->condition = 'conf_id=:conf_id ';
        $criteria->params = array(':conf_id' => $conf->id);
        $criteria->order = 'id desc';
        return $this->findAll($criteria);
    }
    
    public function findAllHanging() {
        $twoDays=2*24*60*60;//  2 дня, 2 days
        $date=time()- $twoDays;
        $criteria = new CDbCriteria;
        $criteria->condition = 'creation_date < :date and completion_date is null ';
        $criteria->params = array(':date' => $date);
        $criteria->order = 'id desc';
        return $this->findAll($criteria);
    }
    
}

?>
